@@ -1,6 +1,5 @@
 # Shortcuts to MIB instrumentation items used internally in SNMP applications
 from pysnmp.smi.error import SmiError, NoSuchInstanceError
-from pysnmp.smi.exval import noSuchInstance
 from pysnmp.entity import config
 
 def getTargetAddr(snmpEngine, snmpTargetAddrName):
@@ -10,10 +9,10 @@ def getTargetAddr(snmpEngine, snmpTargetAddrName):
         'SNMP-TARGET-MIB', 'snmpTargetAddrEntry'
     )
 
-    if 'getTargetAddr' not in snmpEngine.cache:
-        snmpEngine.cache['getTargetAddr'] = { 'id': -1 }
-
-    cache = snmpEngine.cache['getTargetAddr']
+    cache = snmpEngine.getUserContext('getTargetAddr')
+    if cache is None:
+        cache = {'id': -1}
+        snmpEngine.setUserContext(getTargetAddr=cache)
 
     if cache['id'] != snmpTargetAddrEntry.branchVersionId:
         cache['nameToTargetMap'] = {}
@@ -21,18 +20,17 @@ def getTargetAddr(snmpEngine, snmpTargetAddrName):
     nameToTargetMap = cache['nameToTargetMap']
 
     if snmpTargetAddrName not in nameToTargetMap:
-        ( snmpTargetAddrTDomain,
-          snmpTargetAddrTAddress,
-          snmpTargetAddrTimeout,
-          snmpTargetAddrRetryCount,
-          snmpTargetAddrParams ) = mibBuilder.importSymbols(
-                                      'SNMP-TARGET-MIB', 
-                                      'snmpTargetAddrTDomain',
-                                      'snmpTargetAddrTAddress',
-                                      'snmpTargetAddrTimeout',
-                                      'snmpTargetAddrRetryCount',
-                                      'snmpTargetAddrParams'
-                                   )
+        (snmpTargetAddrTDomain,
+         snmpTargetAddrTAddress,
+         snmpTargetAddrTimeout,
+         snmpTargetAddrRetryCount,
+         snmpTargetAddrParams) = mibBuilder.importSymbols(
+             'SNMP-TARGET-MIB', 'snmpTargetAddrTDomain',
+             'snmpTargetAddrTAddress', 'snmpTargetAddrTimeout',
+             'snmpTargetAddrRetryCount', 'snmpTargetAddrParams'
+         )
+        snmpSourceAddrTAddress, = mibBuilder.importSymbols('PYSNMP-SOURCE-MIB', 'snmpSourceAddrTAddress')
+
         tblIdx = snmpTargetAddrEntry.getInstIdFromIndices(snmpTargetAddrName)
 
         try:
@@ -43,7 +41,7 @@ def getTargetAddr(snmpEngine, snmpTargetAddrName):
                 snmpTargetAddrTAddress.name + tblIdx
             ).syntax
             snmpTargetAddrTimeout = snmpTargetAddrTimeout.getNode(
-                 snmpTargetAddrTimeout.name + tblIdx
+                snmpTargetAddrTimeout.name + tblIdx
             ).syntax
             snmpTargetAddrRetryCount = snmpTargetAddrRetryCount.getNode(
                 snmpTargetAddrRetryCount.name + tblIdx
@@ -51,21 +49,28 @@ def getTargetAddr(snmpEngine, snmpTargetAddrName):
             snmpTargetAddrParams = snmpTargetAddrParams.getNode(
                 snmpTargetAddrParams.name + tblIdx
             ).syntax
+            snmpSourceAddrTAddress = snmpSourceAddrTAddress.getNode(
+                snmpSourceAddrTAddress.name + tblIdx
+            ).syntax
         except NoSuchInstanceError:
             raise SmiError('Target %s not configured to LCD' % snmpTargetAddrName)
 
+        transport = snmpEngine.transportDispatcher.getTransport(snmpTargetAddrTDomain)
+
         if snmpTargetAddrTDomain[:len(config.snmpUDPDomain)] == config.snmpUDPDomain:
             SnmpUDPAddress, = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder.importSymbols('SNMPv2-TM', 'SnmpUDPAddress')
-            snmpTargetAddrTAddress = tuple(
+            snmpTargetAddrTAddress = transport.addressType(
                 SnmpUDPAddress(snmpTargetAddrTAddress)
-            )
+            ).setLocalAddress(SnmpUDPAddress(snmpSourceAddrTAddress))
         elif snmpTargetAddrTDomain[:len(config.snmpUDP6Domain)] == config.snmpUDP6Domain:
             TransportAddressIPv6, = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder.importSymbols('TRANSPORT-ADDRESS-MIB', 'TransportAddressIPv6')
-            snmpTargetAddrTAddress = tuple(
+            snmpTargetAddrTAddress = transport.addressType(
                 TransportAddressIPv6(snmpTargetAddrTAddress)
-            )
+            ).setLocalAddress(TransportAddressIPv6(snmpSourceAddrTAddress))
         elif snmpTargetAddrTDomain[:len(config.snmpLocalDomain)] == config.snmpLocalDomain:
-            snmpTargetAddrTAddress = str(snmpTargetAddrTAddress)
+            snmpTargetAddrTAddress = transport.addressType(
+                snmpTargetAddrTAddress
+            )
 
         nameToTargetMap[snmpTargetAddrName] = (
             snmpTargetAddrTDomain,
@@ -87,10 +92,10 @@ def getTargetParams(snmpEngine, paramsName):
         'SNMP-TARGET-MIB', 'snmpTargetParamsEntry'
     )
 
-    if 'getTargetParams' not in snmpEngine.cache:
-        snmpEngine.cache['getTargetParams'] = { 'id': -1 }
-
-    cache = snmpEngine.cache['getTargetParams']
+    cache = snmpEngine.getUserContext('getTargetParams')
+    if cache is None:
+        cache = {'id': -1}
+        snmpEngine.setUserContext(getTargetParams=cache)
 
     if cache['id'] != snmpTargetParamsEntry.branchVersionId:
         cache['nameToParamsMap'] = {}
@@ -98,16 +103,13 @@ def getTargetParams(snmpEngine, paramsName):
     nameToParamsMap = cache['nameToParamsMap']
 
     if paramsName not in nameToParamsMap:
-        ( snmpTargetParamsMPModel,
-          snmpTargetParamsSecurityModel,
-          snmpTargetParamsSecurityName,
-          snmpTargetParamsSecurityLevel ) = mibBuilder.importSymbols(
-                                               'SNMP-TARGET-MIB', 
-                                               'snmpTargetParamsMPModel',
-                                               'snmpTargetParamsSecurityModel',
-                                               'snmpTargetParamsSecurityName',
-                                               'snmpTargetParamsSecurityLevel'
-                                            )
+        (snmpTargetParamsMPModel, snmpTargetParamsSecurityModel,
+         snmpTargetParamsSecurityName,
+         snmpTargetParamsSecurityLevel) = mibBuilder.importSymbols(
+             'SNMP-TARGET-MIB', 'snmpTargetParamsMPModel',
+             'snmpTargetParamsSecurityModel', 'snmpTargetParamsSecurityName',
+             'snmpTargetParamsSecurityLevel'
+         )
 
         tblIdx = snmpTargetParamsEntry.getInstIdFromIndices(paramsName)
 
@@ -127,12 +129,10 @@ def getTargetParams(snmpEngine, paramsName):
         except NoSuchInstanceError:
             raise SmiError('Parameters %s not configured at LCD' % paramsName)
 
-        nameToParamsMap[paramsName] = (
-            snmpTargetParamsMPModel,
-            snmpTargetParamsSecurityModel,
-            snmpTargetParamsSecurityName,
-            snmpTargetParamsSecurityLevel
-        )
+        nameToParamsMap[paramsName] = (snmpTargetParamsMPModel,
+                                       snmpTargetParamsSecurityModel,
+                                       snmpTargetParamsSecurityName,
+                                       snmpTargetParamsSecurityLevel)
 
         cache['id'] = snmpTargetParamsEntry.branchVersionId
 
@@ -140,37 +140,33 @@ def getTargetParams(snmpEngine, paramsName):
 
 def getTargetInfo(snmpEngine, snmpTargetAddrName):
     # Transport endpoint
-    ( snmpTargetAddrTDomain,
-      snmpTargetAddrTAddress,
-      snmpTargetAddrTimeout,
-      snmpTargetAddrRetryCount,
-      snmpTargetAddrParams ) = getTargetAddr(snmpEngine, snmpTargetAddrName)
+    (snmpTargetAddrTDomain,
+     snmpTargetAddrTAddress,
+     snmpTargetAddrTimeout,
+     snmpTargetAddrRetryCount,
+     snmpTargetAddrParams) = getTargetAddr(snmpEngine, snmpTargetAddrName)
 
-    ( snmpTargetParamsMPModel,
-      snmpTargetParamsSecurityModel,
-      snmpTargetParamsSecurityName,
-      snmpTargetParamsSecurityLevel ) = getTargetParams(snmpEngine, snmpTargetAddrParams)
+    (snmpTargetParamsMPModel,
+     snmpTargetParamsSecurityModel,
+     snmpTargetParamsSecurityName,
+     snmpTargetParamsSecurityLevel) = getTargetParams(snmpEngine,
+                                                      snmpTargetAddrParams)
 
-    return ( snmpTargetAddrTDomain,
-             snmpTargetAddrTAddress,
-             snmpTargetAddrTimeout,
-             snmpTargetAddrRetryCount,
-             snmpTargetParamsMPModel,
-             snmpTargetParamsSecurityModel,
-             snmpTargetParamsSecurityName,
-             snmpTargetParamsSecurityLevel )
+    return (snmpTargetAddrTDomain, snmpTargetAddrTAddress,
+            snmpTargetAddrTimeout, snmpTargetAddrRetryCount,
+            snmpTargetParamsMPModel, snmpTargetParamsSecurityModel,
+            snmpTargetParamsSecurityName, snmpTargetParamsSecurityLevel)
 
 def getNotificationInfo(snmpEngine, notificationTarget):
     mibBuilder = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder
 
-    snmpNotifyEntry, = mibBuilder.importSymbols(
-        'SNMP-NOTIFICATION-MIB', 'snmpNotifyEntry'
-    )
+    snmpNotifyEntry, = mibBuilder.importSymbols('SNMP-NOTIFICATION-MIB',
+                                                'snmpNotifyEntry')
 
-    if 'getNotificationInfo' not in snmpEngine.cache:
-        snmpEngine.cache['getNotificationInfo'] = { 'id': -1 }
-
-    cache = snmpEngine.cache['getNotificationInfo']
+    cache = snmpEngine.getUserContext('getNotificationInfo')
+    if cache is None:
+        cache = {'id': -1}
+        snmpEngine.setUserContext(getNotificationInfo=cache)
 
     if cache['id'] != snmpNotifyEntry.branchVersionId:
         cache['targetToNotifyMap'] = {}
@@ -178,12 +174,10 @@ def getNotificationInfo(snmpEngine, notificationTarget):
     targetToNotifyMap = cache['targetToNotifyMap']
 
     if notificationTarget not in targetToNotifyMap:
-        ( snmpNotifyTag,
-          snmpNotifyType ) = mibBuilder.importSymbols(
-                                 'SNMP-NOTIFICATION-MIB', 
-                                 'snmpNotifyTag',
-                                 'snmpNotifyType'
-                             )
+        (snmpNotifyTag,
+         snmpNotifyType) = mibBuilder.importSymbols('SNMP-NOTIFICATION-MIB',
+                                                    'snmpNotifyTag',
+                                                    'snmpNotifyType')
 
         tblIdx = snmpNotifyEntry.getInstIdFromIndices(notificationTarget)
 
@@ -196,7 +190,7 @@ def getNotificationInfo(snmpEngine, notificationTarget):
             ).syntax
 
         except NoSuchInstanceError:
-            raise SmiError('Target %s not configured at LCD' % paramsName)
+            raise SmiError('Target %s not configured at LCD' % notificationTarget)
 
         targetToNotifyMap[notificationTarget] = (
             snmpNotifyTag,
@@ -210,14 +204,13 @@ def getNotificationInfo(snmpEngine, notificationTarget):
 def getTargetNames(snmpEngine, tag):
     mibBuilder = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder
 
-    snmpTargetAddrEntry, = mibBuilder.importSymbols(
-        'SNMP-TARGET-MIB', 'snmpTargetAddrEntry'
-    )
+    snmpTargetAddrEntry, = mibBuilder.importSymbols('SNMP-TARGET-MIB',
+                                                    'snmpTargetAddrEntry')
 
-    if 'getTargetNames' not in snmpEngine.cache:
-        snmpEngine.cache['getTargetNames'] = { 'id': -1 }
-
-    cache = snmpEngine.cache['getTargetNames']
+    cache = snmpEngine.getUserContext('getTargetNames')
+    if cache is None:
+        cache = {'id': -1}
+        snmpEngine.setUserContext(getTargetNames=cache)
 
     if cache['id'] == snmpTargetAddrEntry.branchVersionId:
         tagToTargetsMap = cache['tagToTargetsMap']
@@ -226,17 +219,13 @@ def getTargetNames(snmpEngine, tag):
 
         tagToTargetsMap = cache['tagToTargetsMap']
 
-        ( SnmpTagValue,
-          snmpTargetAddrName,
-          snmpTargetAddrTagList ) = mibBuilder.importSymbols(
-                                        'SNMP-TARGET-MIB',
-                                        'SnmpTagValue',
-                                        'snmpTargetAddrName',
-                                        'snmpTargetAddrTagList'
-                                    )
-        targetNames = []
-        mibNode =  snmpTargetAddrTagList
-        while 1:
+        (SnmpTagValue, snmpTargetAddrName,
+         snmpTargetAddrTagList) = mibBuilder.importSymbols(
+             'SNMP-TARGET-MIB', 'SnmpTagValue', 'snmpTargetAddrName',
+             'snmpTargetAddrTagList'
+         )
+        mibNode = snmpTargetAddrTagList
+        while True:
             try:
                 mibNode = snmpTargetAddrTagList.getNextNode(mibNode.name)
             except NoSuchInstanceError:
