@@ -4,7 +4,7 @@ from pysnmp.smi.indices import OrderedDict, OidOrderedDict
 from pysnmp.smi import error
 from pysnmp import debug
 
-__all__ = [ 'MibViewController' ]
+__all__ = ['MibViewController']
 
 if sys.version_info[0] <= 2:
     import types
@@ -20,7 +20,7 @@ class MibViewController:
         self.lastBuildId = -1
 
     # Indexing part
-    
+
     def indexMib(self):
         if self.lastBuildId == self.mibBuilder.lastBuildId:
             return
@@ -29,12 +29,12 @@ class MibViewController:
 
         MibScalarInstance, = self.mibBuilder.importSymbols(
             'SNMPv2-SMI', 'MibScalarInstance'
-            )
-        
+        )
+
         #
         # Create indices
         #
-        
+
         # Module name -> module-scope indices
         self.__mibSymbolsIdx = OrderedDict()
 
@@ -48,14 +48,14 @@ class MibViewController:
                 r = m.getRevisions()
                 if r:
                     return r[0]
-            else:
-                return "1970-01-01 00:00"
+
+            return "1970-01-01 00:00"
 
         modNames = list(self.mibBuilder.mibSymbols.keys())
         modNames.sort(key=__sortFun)
-            
+
         # Index modules names
-        for modName in [ '' ] + modNames:
+        for modName in [''] + modNames:
             # Modules index
             self.__mibSymbolsIdx[modName] = mibMod = {
                 'oidToLabelIdx': OidOrderedDict(),
@@ -63,7 +63,7 @@ class MibViewController:
                 'varToNameIdx': {},
                 'typeToModIdx': OrderedDict(),
                 'oidToModIdx': {}
-                }
+            }
 
             if not modName:
                 globMibMod = mibMod
@@ -78,7 +78,7 @@ class MibViewController:
                         raise error.SmiError(
                             'Duplicate SMI type %s::%s, has %s' % \
                             (modName, n, mibMod['typeToModIdx'][n])
-                            )
+                        )
                     globMibMod['typeToModIdx'][n] = modName
                     mibMod['typeToModIdx'][n] = modName
                 elif isinstance(v, instanceTypes):
@@ -88,7 +88,7 @@ class MibViewController:
                         raise error.SmiError(
                             'Duplicate MIB variable %s::%s has %s' % \
                             (modName, n, mibMod['varToNameIdx'][n])
-                            )
+                        )
                     globMibMod['varToNameIdx'][n] = v.name
                     mibMod['varToNameIdx'][n] = v.name
                     # Potentionally ambiguous mapping ahead
@@ -99,23 +99,25 @@ class MibViewController:
                 else:
                     raise error.SmiError(
                         'Unexpected object %s::%s' % (modName, n)
-                        )
-            
+                    )
+
         # Build oid->long-label index
         oidToLabelIdx = self.__mibSymbolsIdx['']['oidToLabelIdx']
         labelToOidIdx = self.__mibSymbolsIdx['']['labelToOidIdx']
-        if oidToLabelIdx:
-            prevOid = oidToLabelIdx.keys()[0]
-        else:
-            prevOid = ()
+        prevOid = ()
         baseLabel = ()
         for key in oidToLabelIdx.keys():
             keydiff = len(key) - len(prevOid)
             if keydiff > 0:
-                baseLabel = oidToLabelIdx[prevOid]
-                if keydiff > 1:
-                    baseLabel = baseLabel + key[-keydiff:-1]
-            if keydiff < 0:
+                if prevOid:
+                    if keydiff == 1:
+                        baseLabel = oidToLabelIdx[prevOid]
+                    else:
+                        baseLabel += key[-keydiff:-1]
+                else:
+                    baseLabel = ()
+            elif keydiff < 0:
+                baseLabel = ()
                 keyLen = len(key)
                 i = keyLen-1
                 while i:
@@ -123,9 +125,9 @@ class MibViewController:
                     if k in oidToLabelIdx:
                         baseLabel = oidToLabelIdx[k]
                         if i != keyLen-1:
-                            baseLabel = baseLabel + key[i:-1]
+                            baseLabel += key[i:-1]
                         break
-                    i = i - 1
+                    i -= 1
             # Build oid->long-label index
             oidToLabelIdx[key] = baseLabel + oidToLabelIdx[key]
             # Build label->oid index
@@ -137,18 +139,24 @@ class MibViewController:
             for oid in mibMod['oidToLabelIdx'].keys():
                 mibMod['oidToLabelIdx'][oid] = oidToLabelIdx[oid]
                 mibMod['labelToOidIdx'][oidToLabelIdx[oid]] = oid
-            
+
         self.lastBuildId = self.mibBuilder.lastBuildId
 
     # Module management
-    
-    def getFirstModuleName(self):
+
+    def getOrderedModuleName(self, index):
         self.indexMib()
         modNames = self.__mibSymbolsIdx.keys()
         if modNames:
-            return modNames[0]
+            return modNames[index]
         raise error.SmiError('No modules loaded at %s' % self)
 
+    def getFirstModuleName(self):
+        return self.getOrderedModuleName(0)
+
+    def getLastModuleName(self):
+        return self.getOrderedModuleName(-1)
+
     def getNextModuleName(self, modName):
         self.indexMib()
         try:
@@ -156,7 +164,7 @@ class MibViewController:
         except KeyError:
             raise error.SmiError(
                 'No module next to %s at %s' % (modName, self)
-                )
+            )
 
     # MIB tree node management
 
@@ -172,9 +180,9 @@ class MibViewController:
             return nodeName, nodeName, ()
         oid, label, suffix = self.__getOidLabel(
             nodeName[:-1], oidToLabelIdx, labelToOidIdx
-            )
+        )
         suffix = suffix + nodeName[-1:]
-        resLabel = label + tuple([ str(x) for x in suffix ])
+        resLabel = label + tuple([str(x) for x in suffix])
         if resLabel in labelToOidIdx:
             return labelToOidIdx[resLabel], resLabel, ()
         resOid = oid + suffix
@@ -183,22 +191,20 @@ class MibViewController:
         return oid, label, suffix
 
     def getNodeNameByOid(self, nodeName, modName=''):
-        self.indexMib()        
+        self.indexMib()
         if modName in self.__mibSymbolsIdx:
             mibMod = self.__mibSymbolsIdx[modName]
         else:
-            raise error.SmiError(
-                'No module %s at %s' % (modName, self)
-                )
+            raise error.SmiError('No module %s at %s' % (modName, self))
         oid, label, suffix = self.__getOidLabel(
             nodeName, mibMod['oidToLabelIdx'], mibMod['labelToOidIdx']
-            )
+        )
         if oid == label:
             raise error.NoSuchObjectError(
-                str='Can\'t resolve node name %s::%s at %s' % 
+                str='Can\'t resolve node name %s::%s at %s' %
                 (modName, nodeName, self)
-                )
-        debug.logger & debug.flagMIB and debug.logger('getNodeNameByOid: resolved %s:%s -> %s' % (modName, nodeName, label + suffix))
+            )
+        debug.logger & debug.flagMIB and debug.logger('getNodeNameByOid: resolved %s:%s -> %s.%s' % (modName, nodeName, label, suffix))
         return oid, label, suffix
 
     def getNodeNameByDesc(self, nodeName, modName=''):
@@ -206,15 +212,13 @@ class MibViewController:
         if modName in self.__mibSymbolsIdx:
             mibMod = self.__mibSymbolsIdx[modName]
         else:
-            raise error.SmiError(
-                'No module %s at %s' % (modName, self)
-                )
+            raise error.SmiError('No module %s at %s' % (modName, self))
         if nodeName in mibMod['varToNameIdx']:
             oid = mibMod['varToNameIdx'][nodeName]
         else:
             raise error.NoSuchObjectError(
                 str='No such symbol %s::%s at %s' % (modName, nodeName, self)
-                )
+            )
         debug.logger & debug.flagMIB and debug.logger('getNodeNameByDesc: resolved %s:%s -> %s' % (modName, nodeName, oid))
         return self.getNodeNameByOid(oid, modName)
 
@@ -226,29 +230,34 @@ class MibViewController:
             return self.getNodeNameByOid(nodeName, modName)
         except error.NoSuchObjectError:
             # ...on failure, try as MIB symbol
-            oid, label, suffix = self.getNodeNameByDesc(
-                nodeName[0], modName
-                )
+            oid, label, suffix = self.getNodeNameByDesc(nodeName[0], modName)
             # ...with trailing suffix
-            return self.getNodeNameByOid(
-                    oid + suffix + nodeName[1:], modName
-                    )
-        
-    def getFirstNodeName(self, modName=''):
+            return self.getNodeNameByOid(oid + suffix + nodeName[1:], modName)
+
+    def getOrderedNodeName(self, index, modName=''):
         self.indexMib()
         if modName in self.__mibSymbolsIdx:
             mibMod = self.__mibSymbolsIdx[modName]
         else:
-            raise error.SmiError(
-                'No module %s at %s' % (modName, self)
-                )
+            raise error.SmiError('No module %s at %s' % (modName, self))
         if not mibMod['oidToLabelIdx']:
             raise error.NoSuchObjectError(
                 str='No variables at MIB module %s at %s' % (modName, self)
-                )
-        oid, label = mibMod['oidToLabelIdx'].items()[0]
+            )
+        try:
+            oid, label = mibMod['oidToLabelIdx'].items()[index]
+        except KeyError:
+            raise error.NoSuchObjectError(
+                str='No symbol at position %s in MIB module %s at %s' % (index, modName, self)
+            )
         return oid, label, ()
-        
+
+    def getFirstNodeName(self, modName=''):
+        return self.getOrderedNodeName(0, modName)
+
+    def getLastNodeName(self, modName=''):
+        return self.getOrderedNodeName(-1, modName)
+
     def getNextNodeName(self, nodeName, modName=''):
         oid, label, suffix = self.getNodeName(nodeName, modName)
         try:
@@ -258,21 +267,21 @@ class MibViewController:
         except KeyError:
             raise error.NoSuchObjectError(
                 str='No name next to %s::%s at %s' % (modName, nodeName, self)
-                )
-    
+            )
+
     def getParentNodeName(self, nodeName, modName=''):
         oid, label, suffix = self.getNodeName(nodeName, modName)
         if len(oid) < 2:
             raise error.NoSuchObjectError(
                 str='No parent name for %s::%s at %s' %
                 (modName, nodeName, self)
-                )
+            )
         return oid[:-1], label[:-1], oid[-1:] + suffix
 
     def getNodeLocation(self, nodeName, modName=''):
         oid, label, suffix = self.getNodeName(nodeName, modName)
         return self.__mibSymbolsIdx['']['oidToModIdx'][oid], label[-1], suffix
-    
+
     # MIB type management
 
     def getTypeName(self, typeName, modName=''):
@@ -282,30 +291,34 @@ class MibViewController:
         else:
             raise error.SmiError(
                 'No module %s at %s' % (modName, self)
-                )
+            )
         if typeName in mibMod['typeToModIdx']:
             m = mibMod['typeToModIdx'][typeName]
         else:
             raise error.NoSuchObjectError(
                 str='No such type %s::%s at %s' % (modName, typeName, self)
-                )
+            )
         return m, typeName
-        
-    def getFirstTypeName(self, modName=''):
+
+    def getOrderedTypeName(self, index, modName=''):
         self.indexMib()
         if modName in self.__mibSymbolsIdx:
             mibMod = self.__mibSymbolsIdx[modName]
         else:
-            raise error.SmiError(
-                'No module %s at %s' % (modName, self)
-                )
+            raise error.SmiError('No module %s at %s' % (modName, self))
         if not mibMod['typeToModIdx']:
             raise error.NoSuchObjectError(
                 str='No types at MIB module %s at %s' % (modName, self)
-                )
-        t = mibMod['typeToModIdx'].keys()[0]
+            )
+        t = mibMod['typeToModIdx'].keys()[index]
         return mibMod['typeToModIdx'][t], t
-        
+
+    def getFirstTypeName(self, modName=''):
+        return self.getOrderedTypeName(0, modName)
+
+    def getLastTypeName(self, modName=''):
+        return self.getOrderedTypeName(-1, modName)
+
     def getNextType(self, typeName, modName=''):
         m, t = self.getTypeName(typeName, modName)
         try:
@@ -313,4 +326,4 @@ class MibViewController:
         except KeyError:
             raise error.NoSuchObjectError(
                 str='No type next to %s::%s at %s' % (modName, typeName, self)
-                )
+            )
